USE AdventureWorksDW2012XL;
GO

SET STATISTICS TIME ON;

--DBCC DROPCLEANBUFFERS;

-- Here's a fairly standard query on the (enlarged) FactResellerSales table.
-- We want total sales by product, but only for sales in the European region
-- with a due date ranging from 2009-2012.
-- Dropping and ruilding the columnstore index takes too much time, so I use the
-- IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX hint to mimic behaviour without CS index.
--
-- (I know the query can be simplified to not use a CTE,
--  but this format is required for the next demo,
--  and I elected to use a consistent format.
--  Feel free to rewrite / simplify these and other queries;
--  most such rewrites should not change performance, except where I indicate otherwise).
WITH EuropeanSales
AS (SELECT     r.ProductKey,
               r.SalesAmount
    FROM       dbo.FactResellerSalesXL          AS  r
    INNER JOIN dbo.DimSalesTerritory            AS  s
          ON   s.SalesTerritoryKey               =  r.SalesTerritoryKey
    WHERE      s.SalesTerritoryGroup             = 'Europe'
	AND        r.DueDate                   BETWEEN '20090101' AND '20121231')
SELECT         p.EnglishProductName,
               COUNT(*)                         AS  NumberOfSales,
               SUM(s.SalesAmount)               AS  TotalSalesAmount
FROM           dbo.DimProduct                   AS  p
INNER JOIN     EuropeanSales                    AS  s
      ON       s.ProductKey                      =  p.ProductKey
GROUP BY       p.EnglishProductName
ORDER BY       p.EnglishProductName
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

-- Elapsed times for above query (without columnstore index)
-- Measured as average of three consecutive tests, with no other programs running

-- Execution time with "cold" cache: 217983 ms
-- Execution time with "hot" cache:  139966 ms
go


--DBCC DROPCLEANBUFFERS;

-- But what if we want to include ALL products in the report,
-- including those that didn't sell in the specified period in Europe?
-- Simple - change the join from an INNER to a LEFT (outer) JOIN.
WITH EuropeanSales
AS (SELECT     r.ProductKey,
               r.SalesAmount
    FROM       dbo.FactResellerSalesXL          AS  r
    INNER JOIN dbo.DimSalesTerritory            AS  s
          ON   s.SalesTerritoryKey               =  r.SalesTerritoryKey
    WHERE      s.SalesTerritoryGroup             = 'Europe'
	AND        r.DueDate                   BETWEEN '20090101' AND '20121231')
SELECT         p.EnglishProductName,
               COUNT(*)                         AS  NumberOfSales,
               SUM(s.SalesAmount)               AS  TotalSalesAmount
FROM           dbo.DimProduct                   AS  p
LEFT  JOIN     EuropeanSales                    AS  s
      ON       s.ProductKey                      =  p.ProductKey
GROUP BY       p.EnglishProductName
ORDER BY       p.EnglishProductName
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

-- Elapsed times for above query (without columnstore index)
-- Measured as average of three consecutive tests, with no other programs running

-- Execution times with "cold" cache: 219247 ms
-- Execution times with "hot" cache:  141107 ms
go



--DBCC DROPCLEANBUFFERS;

-- Here is the first query again, but now without a hint;
-- the optimizer will pick the columnstore index,
-- and use batch mode for a large part of the plan.
WITH EuropeanSales
AS (SELECT     r.ProductKey,
               r.SalesAmount
    FROM       dbo.FactResellerSalesXL          AS  r
    INNER JOIN dbo.DimSalesTerritory            AS  s
          ON   s.SalesTerritoryKey               =  r.SalesTerritoryKey
    WHERE      s.SalesTerritoryGroup             = 'Europe'
	AND        r.DueDate                   BETWEEN '20090101' AND '20121231')
SELECT         p.EnglishProductName,
               COUNT(*)                         AS  NumberOfSales,
               SUM(s.SalesAmount)               AS  TotalSalesAmount
FROM           dbo.DimProduct                   AS  p
INNER JOIN     EuropeanSales                    AS  s
      ON       s.ProductKey                      =  p.ProductKey
GROUP BY       p.EnglishProductName
ORDER BY       p.EnglishProductName;

-- Execution time with "cold" cache: 8009 ms =  27 x faster
-- Execution time with "hot" cache:   433 ms = 323 x faster
go

--DBCC DROPCLEANBUFFERS;

-- Here is the second query again, but now without a hint;
-- the optimizer will pick the columnstore index,
-- but batch mode cannot be used because of the outer join.
WITH EuropeanSales
AS (SELECT     r.ProductKey,
               r.SalesAmount
    FROM       dbo.FactResellerSalesXL          AS  r
    INNER JOIN dbo.DimSalesTerritory            AS  s
          ON   s.SalesTerritoryKey               =  r.SalesTerritoryKey
    WHERE      s.SalesTerritoryGroup             = 'Europe'
	AND        r.DueDate                   BETWEEN '20090101' AND '20121231')
SELECT         p.EnglishProductName,
               COUNT(*)                         AS  NumberOfSales,
               SUM(s.SalesAmount)               AS  TotalSalesAmount
FROM           dbo.DimProduct                   AS  p
LEFT  JOIN     EuropeanSales                    AS  s
      ON       s.ProductKey                      =  p.ProductKey
GROUP BY       p.EnglishProductName
ORDER BY       p.EnglishProductName;

-- Execution times with "cold" cache: 8214 ms =  still 27 x faster
-- Execution times with "hot" cache:  4786 ms = "only" 29 x faster
go
